module DSH

  def match_word w
    /([^a-z0-9]|\b)#{w}([^a-z0-9]|\b)/i
  end

  def apply_suffix g, group, suffix
    return g if g =~ /#{suffix}$/
    g == group ? g : "#{g}#{suffix}"
  end

  def apply_prefix g, group, prefix
    return g if g =~ /^#{prefix}/
    g == group ? g : "#{prefix}#{g}"
  end


  def dsh_groups_from_search mappings=node['dsh']['mappings'], nodes=search(:node).map(&:to_hash)
    dsh_groups = Hash.new { |h,k| h[k] = [] }

    nodes.sort_by { |n| n['name'] }.each do |n|
      found_env = false
      node_env = n['environment']

      mappings['environments'].each do |env, use|
        next if found_env
        next unless use
        if env == n['environment']
          found_env = true
          node_env = use if use.is_a?(String)
        end
      end

      unless found_env
        Chef::Log.debug "DSH: Skipping node '#{n['name']}'"
        next
      end

      groups = [ node_env ]

      mappings['names'].each do |name, group|
        groups << group if n['name'] =~ match_word(name)
      end

      begin
        n['roles'].each do |r|
          mappings['roles'].each do |word, role|
            groups << role if r =~ match_word(word)
          end
        end
      rescue
        Chef::Log.warn "DSH: Issues processing node '#{n['name']}'"
        next
      end

      mappings['tags'].each do |tag, use|
        next unless use
        use = tag if use == true
        groups << use if n['tags'] && n['tags'].include?(tag)
      end

      mappings['aggregates'].each do |role, matched_roles|
        matched_roles.each do |matched_role|
          groups << role if groups.include? matched_role
        end
      end

      mappings['excludes'].each do |filter_group, filter|
        next unless filter
        groups.delete_if { |g| g == filter_group }
      end

      host = n[mappings['host_field']]
      host_with_user = host
      host_with_user = "#{mappings['user_name']}@#{host}" if mappings['user_name']

      mappings['shim_user_name'].each do |group, user|
        next unless groups.include? group
        host_with_user = "#{user}@#{host}"
      end

      mappings['suffixes'].each do |group, suffix|
        next unless groups.include? group
        groups += groups.map do |g|
          apply_suffix g, group, suffix
        end
      end

      mappings['prefixes'].each do |group, prefix|
        next unless groups.include? group
        groups += groups.map do |g|
          apply_prefix g, group, prefix
        end
      end

      mappings['suffixes!'].each do |group, suffix|
        next unless groups.include? group
        groups.map! do |g|
          apply_suffix g, group, suffix
        end
      end

      mappings['prefixes!'].each do |group, prefix|
        next unless groups.include? group
        groups.map! do |g|
          apply_prefix g, group, prefix
        end
      end

      groups.uniq.each do |dsh_group|
        dsh_groups[dsh_group] << host_with_user
      end
    end

    return Hash[dsh_groups.sort_by { |g,_| g }]
  end


end



class Chef
  class Recipe
    include DSH
  end

  class Resource
    include DSH
  end
end