#
# Cookbook Name:: bjn_dsh
# Recipe:: default
#
# Copyright (C) 2015 Blue Jeans Network
#

dsh = materialize node['dsh']

mappings = dsh['mappings']

nodes = if Chef::Config[:solo]
  search(:node).map(&:to_hash)
else
  host_field = mappings['host_field']
  keys = {
    'name' => %w[ name ],
    'environment' => %w[ chef_environment ],
    'roles' => %w[ roles ]
  }
  keys[host_field] = [ host_field ]
  partial_search :node, 'name:*', keys: keys
end

dsh_groups = dsh_groups_from_search mappings, nodes

reify_packages dsh['packages']

file dsh['config_path'] do
  config = dsh['config'].map do |k,v|
    "#{k} = #{v}"
  end.join("\n") + "\n"

  content config
end

directory dsh['group_dir'] do
  recursive true
end

ruby_block 'remove-old-dsh-groups' do
  block do
    require 'fileutils'
    group_glob = ::File.join dsh['group_dir'], '*'
    Dir[group_glob].each do |group_path|
      group_name = ::File.basename group_path
      next if dsh_groups.include? group_name
      Chef::Log.warn "DSH: Removing old group '#{group_name}'"
      FileUtils.rm_rf group_path
    end
  end
end

dsh_groups.each do |name, hosts|
  hosts.uniq!
  Chef::Log.debug "DSH: group '%s': %s" % [ name, hosts.join(', ') ]
  group_path = ::File.join dsh['group_dir'], name
  file group_path do
    content hosts.join("\n")
  end
end